package MT::Plugin::EntityRefButton;

use strict;
use MT::Plugin;
@MT::Plugin::EntityRefButton::ISA = qw( MT::Plugin );

use vars qw( $VERSION $PLUGIN_NAME );
$PLUGIN_NAME = 'EntityRefButton';
$VERSION = '0.0.4';

use MT;

my $plugin = MT::Plugin::EntityRefButton->new({
    name => $PLUGIN_NAME,
    version => $VERSION,
    description => '<MT_TRANS phrase=\'This plugin adds a convert button to the entry body and the extended body fields on the edit entry screen, which converts special characters to character entity references.\'>',
    author_name => 'M-Logic, Inc.',
    author_link => 'http://labs.m-logic.jp/',
    doc_link => 'http://labs.m-logic.jp/plugins/entityrefbutton/docs/entityrefbutton.html',
    l10n_class => 'EntityRefButton::L10N',
    blog_config_template => \&template,
    settings => new MT::PluginSettings([
        ['entity_ref_convert_space', { Default => 1 }],
    ]),
});
MT->add_plugin($plugin);
MT->add_callback('MT::App::CMS::AppTemplateSource.edit_entry', 9, $plugin, sub { transform('edit_entry', @_) });
MT->add_callback('MT::App::CMS::AppTemplateSource.bm_entry', 9, $plugin, sub { transform('bm_entry', @_) });
MT->add_callback('MT::App::CMS::AppTemplateParam.edit_entry', 9, $plugin, sub { mod_params('edit_entry', @_) });
MT->add_callback('MT::App::CMS::AppTemplateParam.bm_entry', 9, $plugin, sub { mod_params('bm_entry', @_) });

# NOTE: Lookbehind longer than 255 not implemented in regex;

my %TEMPLATES = (
	       edit_entry => [
			      # script
			      {
			       pattern_s => '\n\n',
			       pattern_e => '//-->\n</script>',
			       replacement => <<'REPLACEMENT_END'
function getCvtSpace() {
    return '<TMPL_VAR NAME=ENTITY_REF_CONVERT_SPACE>';
}

function convertToCharEntityRef(e) {
    if(!canFormat) return;
    var str = getSelected(e);
    if(str) {
        str = str.replace(/&/g, "&amp;");
        str = str.replace(/</g, "&lt;");
        str = str.replace(/>/g, "&gt;");
        str = str.replace(/"/g, "&quot;");
        if(getCvtSpace()) {
            str = str.replace(/ /g, "&nbsp;");
        }
        setSelection(e, str);
    }
    return false;
}

REPLACEMENT_END
,
			      },
			      # body
			      {
			       pattern_s => quotemeta(<<'PATTERN_END')
        write('<img title="<MT_TRANS phrase="Quote">" onclick="return formatStr(document.entry_form.text, \'blockquote\')" src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/quote.gif" alt="<MT_TRANS phrase="Quote">" width="26" height="19" />');
PATTERN_END
,
			       pattern_e => '\s+\}\n\}',
			       replacement => <<'REPLACEMENT_END'
        write('<img title="<TMPL_VAR NAME=ENTITY_REF_BUTTON_LABEL>" onclick="return convertToCharEntityRef(document.entry_form.text)" src="<TMPL_VAR NAME=STATIC_URI>plugins/EntityRefButton/images/formatting-icons/ref.gif" alt="<TMPL_VAR NAME=ENTITY_REF_BUTTON_LABEL>" width="26" height="19" />');
REPLACEMENT_END
,
			      },
			      # extended body
			      {
			       pattern_s => quotemeta(<<'PATTERN_END')
        write('<img title="<MT_TRANS phrase="Quote">" onclick="return formatStr(document.entry_form.text_more, \'blockquote\')" src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/quote.gif" alt="<MT_TRANS phrase="Quote">" width="26" height="19" />');
PATTERN_END
,
			       pattern_e => '\s+\}\n\}',
			       replacement => <<'REPLACEMENT_END'
        write('<img title="<TMPL_VAR NAME=ENTITY_REF_BUTTON_LABEL>" onclick="return convertToCharEntityRef(document.entry_form.text_more)" src="<TMPL_VAR NAME=STATIC_URI>plugins/EntityRefButton/images/formatting-icons/ref.gif" alt="<TMPL_VAR NAME=ENTITY_REF_BUTTON_LABEL>" width="26" height="19" />');
REPLACEMENT_END
			      },
			     ],
	       bm_entry => [
			    # script
			    {
			     pattern_s => '\n\n',
			     pattern_e => '//-->\n</script>',
			     replacement => <<'REPLACEMENT_END'
function getCvtSpace() {
    var ar = new Array();
<TMPL_LOOP NAME=BLOG_LOOP>
    ar[<TMPL_VAR NAME=BLOG_ID>] = '<TMPL_VAR NAME=ENTITY_REF_CONVERT_SPACE>';
</TMPL_LOOP>
    var f = document.entry_form;
    return ar[f.blog_id.options[f.blog_id.selectedIndex].value];
}

function convertToCharEntityRef(e) {
    if(!canFormat) return;
    if(validate) {
        if(!validate(document.entry_form)) {
            return;
        }
    }
    var str = getSelected(e);
    if(str) {
        str = str.replace(/&/g, "&amp;");
        str = str.replace(/</g, "&lt;");
        str = str.replace(/>/g, "&gt;");
        str = str.replace(/"/g, "&quot;");
        if(getCvtSpace()) {
            str = str.replace(/ /g, "&nbsp;");
        }
        setSelection(e, str);
    }
    return false;
}

REPLACEMENT_END
,
			    },
			    # body
			    {
			     pattern_s => quotemeta(<<'PATTERN_END')
        write('<a title="<MT_TRANS phrase="Quote">" href="#" onclick="return formatStr(document.entry_form.text, \'blockquote\')"><img src="<TMPL_VAR NAME=STATIC_URI>images/html-quote.gif" alt="<MT_TRANS phrase="Quote">" width="22" height="16" /></a>');
PATTERN_END
,
			     pattern_e => '\s+\}\n\}',
			     replacement => <<'REPLACEMENT_END'
        write('<a title="<TMPL_VAR NAME=ENTITY_REF_BUTTON_LABEL>" href="#" onclick="return convertToCharEntityRef(document.entry_form.text)"><img src="<TMPL_VAR NAME=STATIC_URI>plugins/EntityRefButton/images/html-ref.gif" alt="<TMPL_VAR NAME=ENTITY_REF_BUTTON_LABEL>" width="22" height="16" /></a>');
REPLACEMENT_END
,
			    },
			    # extended body
			    {
			     pattern_s => quotemeta(<<'PATTERN_END')
        write('<a title="<MT_TRANS phrase="Quote">" href="#" onclick="return formatStr(document.entry_form.text_more, \'blockquote\')"><img src="<TMPL_VAR NAME=STATIC_URI>images/html-quote.gif" alt="<MT_TRANS phrase="Quote">" width="22" height="16" /></a>');
PATTERN_END
,
			     pattern_e => '\s+\}\n\}',
			     replacement => <<'REPLACEMENT_END'
        write('<a title="<TMPL_VAR NAME=ENTITY_REF_BUTTON_LABEL>" href="#" onclick="return convertToCharEntityRef(document.entry_form.text_more)"><img src="<TMPL_VAR NAME=STATIC_URI>plugins/EntityRefButton/images/html-ref.gif" alt="<TMPL_VAR NAME=ENTITY_REF_BUTTON_LABEL>" width="22" height="16" /></a>');
REPLACEMENT_END
,
			    },
			   ],
	      );


sub instance { $plugin; }

sub mod_params {
  my $template_name = shift;
  my ($eh, $app, $param, $tmpl) = @_;
  %$param->{entity_ref_button_label} = $plugin->translate('Convert to character entity reference');
  if($template_name eq 'edit_entry') {
      %$param->{entity_ref_convert_space} = $plugin->entity_ref_convert_space($app->blog->id);
  }
  elsif($template_name eq 'bm_entry') {
    foreach my $item (@{%$param->{blog_loop}}) {
      $item->{entity_ref_convert_space} = $plugin->entity_ref_convert_space($item->{blog_id});
    }
  }
}

sub transform {
  my $template_name = shift;
  my ($eh, $app, $tmpl) = @_;
  my $entries = $TEMPLATES{$template_name};
  foreach my $item (@$entries) {
    my $pattern_s = $item->{pattern_s};
    my $pattern_e = $item->{pattern_e};
    my $replacement = $item->{replacement};
    $$tmpl =~ s!($pattern_s)($pattern_e)!$1$replacement$2!s;
  }
}

sub template {
  my $label = $plugin->translate('Space Conversion:');
  my $description = $plugin->translate('Convert space to non-breaking(&amp;nbsp;) space');
  my $tmpl = <<TEMPLATE_END
    <div class="setting">
    <div class="label">
    <label for="entity_ref_convert_space">$label</label>
    </div>
    <div class="field">
    <p><input type="checkbox" value="1" name="entity_ref_convert_space" id="entity_ref_convert_space"<TMPL_IF NAME=ENTITY_REF_CONVERT_SPACE> checked="checked"</TMPL_IF>>$description</p>
    </div>
    </div>
TEMPLATE_END
}

sub entity_ref_convert_space {
  my $plugin = shift;
  my ($blog_id) = @_;
  my %plugin_param;

  $plugin->load_config(\%plugin_param, 'blog:'.$blog_id);
  my $value = $plugin_param{entity_ref_convert_space};
  unless ($value) {
    $plugin->load_config(\%plugin_param, 'system');
    $value = $plugin_param{entiry_ref_convert_space};
  }
  $value;
}

1;

__END__
